;+
; NAME:
;     mc_apotspecohwavecal
;
; PURPOSE:
;     To wavelength calibration APO TripleSpec data using OH lines.
;
; CATEGORY:
;     Spectroscopy
;
; CALLING SEQUENCE:
;     mc_apotspecohwavecal,parent,[arcfiles],[flat],[arconame],[wconame],$
;                          DARK=dark,CANCEL=cancel
;
; INPUTS:
;     parent - The widget ID of the parent widget into which this one
;              is added.
;
; OPTIONAL INPUTS:
;     arcfiles - An array of the full path names of the arc file names.
;     flat     - The full path name of the flat field.
;     arconame - The output name of the combined arc (without the .fits).
;     wconame  - The output name of the wavecal file (without the .fits).
;
; KEYWORD PARAMETERS:
;     DARKFILE - If given, a dark is subtracted from the combined arc
;                image first.
;     CANCEL - Set on return if there is a problem.
;
; OUTPUTS:
;     Either a widget is created or arconame.fits and wconame.fits
;     files are written to disk.
;
; OPTIONAL OUTPUTS:
;     None
;
; COMMON BLOCKS:
;     xspextool_state
;
; SIDE EFFECTS:
;     None
;
; RESTRICTIONS:
;     None
;
; PROCEDURE:
;      The program both creates a widget that fits into Spextool but
;      also does the wavelength calibration if called with additional 
;      parametesr.
;
; EXAMPLE:
;      mc_apotspecwavecal,widget_id will create a widget.
;
; MODIFICATION HISTORY:
;      2008 - Written by M. Cushing, Institute for Astronomy,
;             University of Hawaii
;-
;
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro tspecwavecal_createwavecal,arcfiles,flatname,arconame,wconame, $
                               DARKFILE=darkfile,CANCEL=cancel

  cancel = 0
  
  common xspextool_state
  
  mc_readflat,flatname,flat,ncols,nrows,obsmode,slith_pix,slith_arc,slitw_pix, $
              slitw_arc,rp,norders,orders,edgecoeffs,xranges,rms,rotation, $
              edgedeg,CANCEL=cancel
  if cancel then return
  
;  Read images.
  
  xspextool_message, 'Combining the arcs...'
  mc_readapotspecfits,arcfiles,data,hdrinfo,var,ROTATE=rotation, $
                      KEYWORDS=state.r.keywords,NIMAGES=nimages,CANCEL=cancel
  if cancel then return
  
  arccombstat = 6
  if nimages ne 1 then begin
     
     arc = median(data,/EVEN,DIMENSION=3)
     
  endif else arc = reform(data)

;  Get Dark if necessary

  if n_elements(DARKFILE) ne 0 then begin

     xspextool_message, 'Subtracting the dark frame...'
     dark = rotate(readfits(darkfile),rotation)
     arc = temporary(arc)-dark
     darkfile = file_basename(darkfile)

  endif else darkfile = 'None'

;  Clean the bad pixels
  
  arc = fixbdpx_median(arc,rotate(*state.d.mask,rotation),edgecoeffs,norders,$
                       xranges,20,CANCEL=cancel)
  if cancel then return
  
  newhdr = hdrinfo[0]
  junk = tag_exist(hdrinfo[*].vals,state.r.irafname,INDEX=idx)
  newhdr.vals.(idx) = arconame+'.fits'
 
  history='This arc was created by '+state.r.combstats[6]+$
          ' combining '+strjoin(hdrinfo[*].vals.(idx), ', ')+'.'
 
  mc_writearc,mc_unrotate(arc,rotation),state.r.calpath+arconame+'.fits', $
              newhdr,state.w.version,HISTORY=history,CANCEL=cancel
  if cancel then return
 
;  Update message.
 
  xspextool_message, 'Wrote arc to '+state.r.calpath+arconame+'.fits'
 
  ximgtool,arc,WID=wid,GROUP_LEADER=state.w.xspextool_base,/AUTORANGE, $
           BUFFER=1,ZWIN=0,STDIMAGE=state.r.stdimage, $
           PLOTWINSIZE=state.r.plotwinsize
  
  xspextool_message, 'Wavelength Calibrating...'
  
  fitthresh = cfld(state.w.wavecalthresh_fld,4,/EMPTY,CANCEL=cancel)
  if cancel then return

;  Read cal files and get standard arc spectra for X-correlation.
  
  file = filepath('OHXD.fits',ROOT=state.r.packagepath,SUB='data')
     
  mc_rdapotspecwavecal,file,xcor_spectra,xcor_orders,dispdeg,w2pcoeffs, $
                       CANCEL=cancel
  if cancel then return
  
;  Create trace and extract the data
  
  aprad = 5
  posarc = fltarr(1,norders)
  posarc[*] = slith_arc/2.
  tracecoeffs = mc_aptotrace(edgecoeffs,slith_arc,posarc,xranges,aprad, $
                             edgedeg,nrows,CANCEL=cancel)
  if cancel then return

  a = obj_new('oplotap')
  a->set, xranges,edgecoeffs,tracecoeffs,1,slith_arc,rebin([aprad],1,norders)
  
  ximgtool,arc,WID=wid,GROUP_LEADER=state.w.xspextool_base,/AUTORANGE, $
           BUFFER=1,ZWIN=0,STDIMAGE=state.r.stdimage, $
           PLOTWINSIZE=state.r.plotwinsize,OPLOT=a
  


  wavespec = extractspec_xs(arc,fltarr(ncols,nrows)+1.,edgecoeffs,$
                            tracecoeffs,norders,1,xranges,slith_arc,[aprad], $
                            /UPDATE,WIDGET_ID=state.w.xspextool_base,$
                            CANCEL=cancel)
  if cancel then return

;  Do the cross-correlation.
  
  match,orders,xcor_orders,idx_arc,idx_xcor

  start = xranges[0,idx_arc[0]]
  stop  = xranges[1,idx_arc[0]]
  ncols = stop-start+1

  arc_x = findgen(stop-start+1)+start
  arc_y = reform(wavespec[*,0,idx_arc[0]])

  xcor_x  = reform(xcor_spectra[start:stop,0,idx_xcor[0]])
  xcor_y  = reform(xcor_spectra[start:stop,1,idx_xcor[0]])

;  Protect against NaNs
  
  z = where(finite(arc_y) eq 0,count)
  if count ne 0 then arc_y[z] = 0.0 
  
  z = where(finite(xcor_y) eq 0,count)
  if count ne 0 then xcor_y[z] = 0.0 
  
  if state.r.autoxcorr then begin
     
     lag = findgen(ncols)-fix(ncols/2.)
     corr = c_correlate(xcor_y,arc_y,lag)
     max = max(corr,idx)
     
     fit = gaussfit(lag[idx-20:idx+20],corr[idx-20:idx+20],a,$
                    NTERMS=4)
     offset = a[1]
     if state.r.plotautoxcorr then begin
        
        window, /FREE
        wid = !d.window
        !p.multi[0] = 2
        !p.multi[2] = 2
        
        plot, lag,corr,/xsty,/ysty
        plot,lag[idx-20:idx+20],corr[idx-20:idx+20],/xsty,/ysty,$
             TITLE='!5Offset = '+strtrim(offset,2)
        oplot, lag[idx-20:idx+20],fit,color=2
        
        result = dialog_message('Is this ok?',/QUESTION)
        if strtrim(result,2) eq 'No' then begin
           
           wdelete, wid
           goto, manual1
           
        endif else wdelete, wid
        
     endif
     message = 'Cross-correlation offset = '+strtrim(offset,2)+' pixels.'
     xspextool_message, message 
     
  endif else begin 
     
     manual1:
     beep
     message = 'Please Identify the same line in each spectrum...'
     xspextool_message,message
     xgetoffset,xcor_x,xcor_y,arc_x,arc_y,offset,ORDER=$
                total(xcor_orders[idx_xcor[0]])
     message = 'Cross-correlation offset = '+$
               strtrim(offset,2)+' pixels.'
     xspextool_message, message 
     
  endelse

;  Get lines
     
  file = filepath('OHll.dat',ROOT=state.r.packagepath,SUB='data')
  mc_rdohll,file,plines,RES=rp,THRESH=3,CANCEL=cancel

  spectra = fltarr(ncols,3,norders)
  disp    = fltarr(norders)

  nlines = intarr(norders)
  rms = dblarr(norders)
  for i = 0,norders-1 do begin

     xglines = poly(plines,w2pcoeffs[*,i])+offset

     result = mc_wavecal1d(arc_x,reform(wavespec[*,0,i]),xranges[*,i],plines, $
                           xglines,slitw_pix,dispdeg[i],fitthresh, $
                           RMS=r,NLINES=n,PLOTLINEFIND=state.r.plotlinefind, $
                           CANCEL=cancel)
     print, i,cancel
     if cancel then return
     spectra[*,0,i] = result
     spectra[*,1,i]= wavespec[*,0,i]
;     if i eq 4 then print, wavespec[*,0,i]
     spectra[*,2,i]= sqrt(wavespec[*,1,i])
     if state.r.errorprop eq 0 then spectra[*,2,i] = 1.

     coeff = poly_fit1d(arc_x,result,1,/SILENT)
     disp[i] = coeff[1]

     nlines[i] = n
     rms[i] = r

  endfor

  xspextool_message,'The number of lines found in order '+ $
                    strtrim(orders,2)+' is: '+strtrim(nlines,2)+'.'
  
  history = 'The RMS deviations for each order are '+$
                    strjoin(strtrim(mc_sigfig(rms*1e4,3),2), ', ')+' Angstroms.'
  xspextool_message,'The RMS deviations for each order are '+$
                    strjoin(strtrim(mc_sigfig(rms*1e4,3),2), ', ')+' Angstroms.'
  
  bad = where(nlines le 2,cnt)
  if cnt ne 0 then begin
     
     ok = dialog_message(strjoin(strtrim(nlines[bad],2),',')+ $
                         ' lines were found in orders '+ $
                         strjoin(strtrim(orders[bad],2),',')+'.', $
                         DIALOG_PARENT=state.w.xspextool_base, $
                         /INFORMATION)

     endif


  yunits='DN / s'
  xunits= 'um'
  
  xtitle='!7k!5 (!7l!5m)'
  ytitle='f (!5DN s!u-1!N)'
  
  
  junk = tag_exist(newhdr.vals,state.r.irafname,INDEX=idx)
  newhdr.vals.(idx) = wconame+'.fits'
  
  appos = fltarr(1,norders)
  appos[0,*] = slith_arc/2.

  mc_writespec,spectra,state.r.calpath+wconame+'.fits',arconame+'.fits', $
               darkfile,file_basename(flatname),1,fix(orders),start,stop, $
               newhdr,appos,[1],obsmode,slith_pix,slith_arc,slitw_pix, $
               slitw_arc,rp,xunits,yunits,xtitle,ytitle,state.r.instr, $
               state.w.version,DISP=disp,RMS1d=rms,RES=rp

  return

;  for j = 0, norders-1 do begin
;     
;     result = poly2d(arc_x,replicate(fix(orders[j]),n_elements(arc_x)),dispdeg,$
;                     ordrdeg,p2lcoeffs)
;     spectra[*,0,j] = result*(float(homeorder)/float(orders[j]))
;     
;     spectra[*,1,j]= wavespec[*,0,j]
;     spectra[*,2,j]= sqrt(wavespec[*,1,j])
;     
;     if state.r.errorprop eq 0 then spectra[*,2,j] = 1.
;     
;     coeff = poly_fit1d(arc_x,spectra[*,0,j],1,/SILENT)
;     disp[j] = coeff[1]
;     
;  endfor
;  
;  res    = resppix/slitw_pix
;  
;  yunits='DN / s'
;  xunits= 'um'
;  
;  xtitle='!7k!5 (!7l!5m)'
;  ytitle='f (!5DN s!u-1!N)'
;  
;  
;  arc = readfits(state.r.calpath+arconame+'.fits',hdr,/SILENT)
;  hdrinfo = gethdrinfo(hdr,state.r.keywords)
;  
;  junk = tag_exist(hdrinfo.vals,state.r.irafname,INDEX=idx)
;  hdrinfo.vals.(idx) = wconame+'.fits'
;  
;  appos = fltarr(1,norders)
;  appos[0,*] = slith_arc/2.
;
;  mc_writespec,spectra,state.r.calpath+wconame,arconame+'.fits','None', $
;               file_basename(flat),1,fix(orders),0,ncols-1,newhdr,appos, $
;               [1],obsmode,slith_pix,slith_arc,slitw_pix,slitw_arc,rp,xunits, $
;               yunits,xtitle,ytitle,state.r.instr,state.w.version,RMS=rms, $
;               RES=rp
  
getout:

end
;
;******************************************************************************
;
pro tspecwavecal_doit,w,CANCEL=cancel

  cancel = 0

  common xspextool_state

  ;  Check filemode.

  index    = (state.r.filereadmode eq 'Index') ? 1:0
  filename = (state.r.filereadmode eq 'Filename') ? 1:0
  
  if index then prefix = cfld(state.w.iprefix_fld,7,/EMPTY,CANCEL=cancel)
  if cancel then return
  
;  Get user inputs.
  
  arcfiles = cfld(w.arcimages_fld,7,/EMPTY,CANCEL=cancel)
  if cancel then return
  arcfiles = fsextract(arcfiles,INDEX=index,FILENAME=filename,CANCEL=cancel)
  if cancel then return
  
  arcfiles = mkfullpath(state.r.datapath,arcfiles,INDEX=index, $
                        FILENAME=filename,NI=state.r.nint,PREFIX=prefix, $
                        SUFFIX='*.fits',WIDGET_ID=state.w.xspextool_base, $
                        /EXIST,CANCEL=cancel)
  if cancel then return

;  darkname = cfld(w.dark_fld,7,CANCEL=cancel)
;  if cancel then return
;  if darkname ne '' then $
;     darkfile = cfile(state.r.calpath+darkname, $
;                      WIDGET_ID=state.w.xspextool_base,CANCEL=cancel)
;  if cancel then return

  flatname = cfld(w.arcflat_fld,7,/EMPTY,CANCEL=cancel)
  if cancel then return
  flatfile = cfile(state.r.calpath+flatname,WIDGET_ID=state.w.xspextool_base,$
                   CANCEL=cancel)
  if cancel then return
  
  arconame = cfld(w.arconame_fld,7,/EMPTY,CANCEL=cancel)
  if cancel then return
  
  wconame = cfld(w.wconame_fld,7,/EMPTY,CANCEL=cancel)
  if cancel then return
  
  tspecwavecal_createwavecal,arcfiles,flatfile,arconame,wconame,CANCEL=cancel
  
end
;
;*****************************************************************************
;
;------------------------------ Event Handler -------------------------------
;
;*****************************************************************************
;
pro tspecwavecal_event,event

  common xspextool_state

  widget_control, event.handler, GET_UVALUE = w, /NO_COPY
  widget_control, event.id,  GET_UVALUE = uvalue
  widget_control, /HOURGLASS

  case uvalue of 

     'Arc Flat Field Button': begin
        
        path =dialog_pickfile(DIALOG_PARENT=state.w.xspextool_base,$
                              PATH=state.r.calpath,/MUST_EXIST,$
                              FILTER='*.fits',/FIX_FILTER)
        
        if path ne '' then widget_control,w.arcflat_fld[1], $
                                          SET_VALUE=file_basename(path)
        
     end

     'Arc Images Button': begin
        
        path= dialog_pickfile(DIALOG_PARENT=state.w.xspextool_base,$
                              /MUST_EXIST,PATH=state.r.datapath,$
                              FILTER=['*.fits'],/FIX_FILTER,$
                              /MULTIPLE_FILES)
        
        if path[0] ne '' then $
           widget_control,w.arcimages_fld[1],$
                          SET_VALUE=strjoin(file_basename(path),',',/SINGLE)
        
     end

     'Dark Button': begin
        
        path =dialog_pickfile(DIALOG_PARENT=state.w.xspextool_base,$
                              PATH=state.r.calpath,/MUST_EXIST,$
                              FILTER='*.fits',/FIX_FILTER)
        
        if path ne '' then $
           widget_control,w.dark_fld[1],$
                          SET_VALUE =strjoin(file_basename(path),',',/SINGLE)

     end

     'Make Wavecal': tspecwavecal_doit,w,CANCEL=cancel

  endcase

  widget_control, event.handler, SET_UVALUE=w, /NO_COPY

end
;
;*****************************************************************************
;
;------------------------------ Main Program -------------------------------
;
;*****************************************************************************
;
pro mc_apotspecohwavecal,parent,arcfiles,flat,arconame,wconame, $
                         DARKFILE=darkfile,CANCEL=cancel
  
  if n_params() gt 1 then begin
     
     tspecwavecal_createwavecal,arcfiles,flat,arconame,wconame, $
                                DARKFILE=darkfile,CANCEL=cancel
     return
     
  endif


  cancel = 0

  w = {arcimages_fld:[0L,0L],$
       arcflat_fld:[0L,0L],$
       arconame_fld:[0L,0L],$
       arcskyrow:0L,$
       dark_fld:[0L,0L],$
       wconame_fld:[0L,0L]}

  getfonts,buttonfont,textfont

  row_base = widget_base(parent,$
                         /ROW)

     col1_base = widget_base(row_base,$
                             /COLUMN,$
                             /FRAME)
  
        row = widget_base(col1_base,$
                          /ROW,$
                          /BASE_ALIGN_CENTER)
        
           button = widget_button(row,$
                                  FONT=buttonfont,$
                                  VALUE='Sky Images',$
                                  UVALUE='Arc Images Button')
           
           fld = coyote_field2(row,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE=':',$
                               UVALUE='Arc Images Field',$
                               XSIZE=30,$
;                               VALUE='ArgonOn001.fits',$
                               TEXTID=textid) 
           w.arcimages_fld = [fld,textid]
           
;           row = widget_base(col1_base,$
;                             /ROW,$
;                             /BASE_ALIGN_CENTER)
;           
;              button = widget_button(row,$
;                                     FONT=buttonfont,$
;                                     VALUE='Full Dark Name',$
;                                     UVALUE='Dark Button')
;              
;              fld = coyote_field2(row,$
;                                  LABELFONT=buttonfont,$
;                                  FIELDFONT=textfont,$
;                                  TITLE=':',$
;                                  UVALUE='Dark Field',$
;                                  XSIZE=15,$
;                                  TEXTID=textid) 
;              w.dark_fld = [fld,textid]
              
        row = widget_base(col1_base,$
                          /ROW,$
                          /BASE_ALIGN_CENTER)
        
           button = widget_button(row,$
                                  FONT=buttonfont,$
                                  VALUE='Full Flat Name',$
                                  UVALUE='Arc Flat Field Button')
           
           fld = coyote_field2(row,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE=':',$
                               UVALUE='Arc Flat Field',$
                               XSIZE=15,$
;                               VALUE='nFlat.fits',$
                               TEXTID=textid) 
           w.arcflat_fld = [fld,textid]
           
        fld = coyote_field2(col1_base,$
                            LABELFONT=buttonfont,$
                            FIELDFONT=textfont,$
                            TITLE='Output Arc Name:',$
                            UVALUE='Output Arc Name',$
                            XSIZE=15,$
;                            VALUE='test',$
                            TEXTID=textid) 
        w.arconame_fld = [fld,textid]
        
        fld = coyote_field2(col1_base,$
                            LABELFONT=buttonfont,$
                            FIELDFONT=textfont,$
                            TITLE='Output Wavecal Name:',$
                            UVALUE='Output Wavecal Name',$
                            XSIZE=15,$
;                            VALUE='wtest',$
                            TEXTID=textid) 
        w.wconame_fld = [fld,textid]
        
     button = widget_button(col1_base,$
                            FONT=buttonfont,$
                            VALUE='Make Wavecal',$
                            UVALUE='Make Wavecal')

; Start the Event Loop. This will be a non-blocking program.

  XManager, 'xmc_spexarc', $
            row_base, $
            EVENT_HANDLER='tspecwavecal_event',$
            /NO_BLOCK

; Put state variable into the user value of the top level base.

  widget_control, row_base, SET_UVALUE=w, /NO_COPY

end
